//==============================================================================
/**
 * @file    main.cpp
 * @brief   エントリーポイント
 * @author  H.I.
 * @date    2012-08-23
 * @version $Id$
 */
//==============================================================================



//==============================================================================
// include
//==============================================================================
#include <iostream>
#include <fstream>
#include <string>
#include <cstdlib>
#include "common.hpp"
#include "xml.hpp"
#include "str_to_number.hpp"


#if defined(CONVERT_STOP_TAG)
#include "stop_as3_inc.cpp"

//==============================================================================
/**
 * XML - AS3書き出し用スペシャライズノード
 */
//==============================================================================
class clXMLNodeSpecial : public hil::utility::clXMLNode{
public:
  typedef hil::utility::clXMLNode super_type;
  
public:
  //-----------------------------------------------------------------------
  /**
   * コンストラクタ
   */
  //-----------------------------------------------------------------------
  clXMLNodeSpecial()
  : hil::utility::clXMLNode(),
    m_frame(1)
  {
    setType(hil::utility::clXMLNode::TYPE_ELEMENT);
  }
  
  //-----------------------------------------------------------------------
  /**
   * デストラクタ
   */
  //-----------------------------------------------------------------------
  virtual ~clXMLNodeSpecial(){}
  
  //-----------------------------------------------------------------------
  /**
   * 表示
   *
   * @param[ in ] hstream : 出力先ストリーム
   * @param[ in ] hindent : インデント
   */
  //-----------------------------------------------------------------------
  virtual void print(std::ostream* hstream=&std::cout, int hindent=0) const
  {
    std::string indent(hindent, ' ');
    
    // AS3コード前半
    for (size_t i = 0 ;
        i < (sizeof(StopAS3FrontString) / sizeof(StopAS3FrontString[0])) ; ++i) {
      (*hstream) << indent << StopAS3FrontString[i] << std::endl;
    }
    // フレーム数
    (*hstream) << indent << 
        "                  <OpPushByte value=\"" << m_frame << "\"/>" << std::endl;
    // AS3コード後半
    for (size_t i = 0 ;
        i < (sizeof(StopAS3RearString) / sizeof(StopAS3RearString[0])) ; ++i) {
      (*hstream) << indent << StopAS3RearString[i] << std::endl;
    }
  }
  
  
  void SetFrame(int hframe) { m_frame = hframe; }
  
private:
  int m_frame;
};
#endif



//==============================================================================
/**
 * エントリーポイント
 */
//==============================================================================
int main(int argc, char* argv[])
{
  if (argc < 3) return EXIT_FAILURE;
  
  hil::utility::clXML xml(argv[1]);
  
  if (xml.isLoad() == false) {
    std::cout << "Can't Open File : " << argv[1] << std::endl;
    return EXIT_FAILURE;
  }
  
  bool swf_tag = false;
  bool file_attribute_tag = false;
  bool convert_stop_tag = false;
  
  // swfタグのバージョン変更
  {
    hil::utility::clXMLNode* node = xml.find("swf");
    if (node) {
      hil::utility::clXMLNode::attribute_type* attr(node->searchAttribute("version"));
      if (attr) {
        if ((attr->second == "6") ||
            (attr->second == "7") ||
            (attr->second == "8")) {
          attr->second = std::string("10");
          swf_tag = true;
          std::cout << "swf tag changed" << std::endl;
        }
      }
    }
  }
  
  // FileAttributesタグのallowABC
  {
    hil::utility::clXMLNode* node = xml.find("FileAttributes");
    if (node) {
      hil::utility::clXMLNode::attribute_type* attr(node->searchAttribute("allowABC"));
      if (attr == NULL) {
        const std::vector<hil::utility::clXMLNode::attribute_type>& attr_list(node->getAttribute());
        std::vector<hil::utility::clXMLNode::attribute_type> attr_list_new;
        
        attr_list_new.resize(attr_list.size() + 1U);
        std::copy(attr_list.begin(), attr_list.end(), attr_list_new.begin());
        
        *attr_list_new.rbegin() = hil::utility::clXMLNode::attribute_type("allowABC", "1");
        
        node->setAttribute(attr_list_new);
        file_attribute_tag = true;
        std::cout << "FileAttributes tag changed" << std::endl;
      }
    }
  }
  
#if defined(CONVERT_STOP_TAG)
  // アクションタグがあるか
  {
    hil::utility::clXMLNode* header = xml.find("Header");
    hil::utility::clXMLNode* node = xml.find("FileAttributes");
    if (node == 0)  node = xml.find("SetBackgroundColor");
    if (node && header && (xml.find("DoABCDefine") == 0)) {
      // 最大フレーム数取得
      clXMLNodeSpecial* as3_node = new clXMLNodeSpecial();
      hil::utility::clXMLNode::attribute_type* frame =
          header->searchAttribute("frames");
      if (frame) {
        int f = 1;
        if (hil::common::StringToNumber(&f, frame->second.c_str())) {
          as3_node->SetFrame(std::max(f-1, 1));
        }
      }
      
      // 力技
      as3_node->m_delete = true;
      as3_node->m_parent = node->m_parent;
      as3_node->m_next = node->m_next;
      node->m_next = as3_node;
      
      convert_stop_tag = true;
      std::cout << "set stop AS3 tags" << std::endl;
    }
  }
#endif
  
  
  if ((swf_tag && file_attribute_tag) || convert_stop_tag) {
    // 変更入っていたら出力
    std::string filepath(argv[2]);
    filepath += ".tmp";
    xml.exportFile(filepath);
    
    // ファイル先頭のxmlバージョンとエンコード設定が正常に出力されていないので
    {
      std::ifstream inf(filepath.c_str(), std::ios::binary);
      std::ofstream of(argv[2], std::ios::binary);
      char str[2048];
      
      inf.getline(str, 2048, '>');
      
      size_t start = inf.tellg();
      inf.seekg(0, std::ios::end);
      size_t end = inf.tellg();
      inf.seekg(start, std::ios::beg);
      
      size_t buf_size = end - start;
      char* buf = new char[buf_size+1];
      inf.read(buf, buf_size);
      buf[buf_size] = '\0';
      of << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << std::endl;
      of.write(buf, buf_size);
      
      delete[] buf;
    }
    std::system((std::string("del \"") + filepath + std::string("\"")).c_str());
    
  } else {
    return EXIT_FAILURE;
  }
  
  
  
  return EXIT_SUCCESS;

}
