//==============================================================================
/**
 * @file    str_to_number.hpp
 * @brief   文字列から数字に
 * @author  H.I.
 * @date    2011-02-19
 * @version $Id: str_to_number.hpp 20 2011-03-27 09:52:29Z blue $
 */
//==============================================================================


#ifndef HIL_STR_TO_NUMBER_HPP
#define HIL_STR_TO_NUMBER_HPP


//==============================================================================
// include
//==============================================================================
#include <iostream>
#include "common.hpp"

HIL_BEGIN_NAMESPACE;

namespace common {

//==============================================================================
// define
//==============================================================================

//==============================================================================
/// 2進数変換
//==============================================================================
class clBinalNumber {
public:
  static const int DecimalRate = 2;    ///< 10進数への変換レート
  
  //-----------------------------------------------------------------------
  /**
   * 文字を変換する
   *
   * @param[ out ]  hnumber     : 変換した数値
   * @param[ in ]   hchr        : 変換する文字
   * @return 正常に変換できたか
   */
  //-----------------------------------------------------------------------
  static bool convert(int* hnumber, char hchr) {
    if (hchr < '0') return false;
    if (hchr > '1') return false;
    
    if (hnumber)  *hnumber = hchr - '0';
    return true;
  }
};

//==============================================================================
/// 8進数変換
//==============================================================================
class clOctalNumber {
public:
  static const int DecimalRate = 8;    ///< 10進数への変換レート
  
  //-----------------------------------------------------------------------
  /**
   * 文字を変換する
   *
   * @param[ out ]  hnumber     : 変換した数値
   * @param[ in ]   hchr        : 変換する文字
   * @return 正常に変換できたか
   */
  //-----------------------------------------------------------------------
  static bool convert(int* hnumber, char hchr) {
    if (hchr < '0') return false;
    if (hchr > '7') return false;
    
    if (hnumber)  *hnumber = hchr - '0';
    return true;
  }
};

//==============================================================================
/// 10進数変換
//==============================================================================
class clDecimalNumber {
public:
  static const int DecimalRate = 10;    ///< 10進数への変換レート
  
  //-----------------------------------------------------------------------
  /**
   * 文字を変換する
   *
   * @param[ out ]  hnumber     : 変換した数値
   * @param[ in ]   hchr        : 変換する文字
   * @return 正常に変換できたか
   */
  //-----------------------------------------------------------------------
  static bool convert(int* hnumber, char hchr) {
    if (hchr < '0') return false;
    if (hchr > '9') return false;
    
    if (hnumber)  *hnumber = hchr - '0';
    return true;
  }
};

//==============================================================================
/// 16進数変換
//==============================================================================
class clHexadecimalNumber {
public:
  static const int DecimalRate = 16;    ///< 10進数への変換レート
  
  //-----------------------------------------------------------------------
  /**
   * 文字を変換する
   *
   * @param[ out ]  hnumber     : 変換した数値
   * @param[ in ]   hchr        : 変換する文字
   * @return 正常に変換できたか
   */
  //-----------------------------------------------------------------------
  static bool convert(int* hnumber, char hchr) {
    if ((hchr >= 'A') && (hchr <= 'F')) {
      if (hnumber)  *hnumber = (hchr - 'A') + 10;
      return true;
    }
    if ((hchr >= 'a') && (hchr <= 'f')) {
      if (hnumber)  *hnumber = (hchr - 'a') + 10;
      return true;
    }
    if (hchr < '0') return false;
    if (hchr > '9') return false;
    
    if (hnumber)  *hnumber = hchr - '0';
    return true;
  }
};



//-----------------------------------------------------------------------
/**
 * 文字列を整数型に変換する
 *
 * @param[ out ]  hnumber     : 変換した数値
 * @param[ in ]   hstr        : 変換する文字列
 * @param[ in ]   hstr_size   : hstrの読み込むサイズ
 * @return 正常に変換できたか
 */
//-----------------------------------------------------------------------
extern bool StringToNumber(int* hnumber, const char* hstr,
    size_t hstr_size=0xffffffff);
    
//-----------------------------------------------------------------------
/**
 * 文字列を浮動小数点型に変換する
 *
 * @param[ out ]  hnumber     : 変換した数値
 * @param[ in ]   hstr        : 変換する文字列
 * @param[ in ]   hstr_size   : hstrの読み込むサイズ
 * @return 正常に変換できたか
 */
//-----------------------------------------------------------------------
extern bool StringToFloat(float* hnumber, const char* hstr,
    size_t hstr_size=0xffffffff);


} // namespace common

HIL_END_NAMESPACE;

#endif


